#!/bin/sh
#
# Displays the status of the SAS EP process on each host of a
# Greenplum database. This script should be executed from the
# database master node.
#
# Usage: 
#    ShowSASEPStatus.sh [-verbose] [-database <database>]
#

verbose=""
database=""
mycommand=$0

pid_file="/var/tmp/tkgpmain.pid"
ep_process="tkgpmain"

product=SASEPForGreenplum
long_product="SAS Embedded Process for Greenplum"

#=====================================================================
# Handle passed in arguments
#=====================================================================

while [ "$#" -gt 0 ]
do
    if [ $1 = "-verbose" ] ; then
        verbose="true"
    elif [ $1 = "-database" ] ; then
        shift
        if [ "$#" -gt 0 ] ; then
            database=$1
        fi
    fi
    if [ "$#" -gt 0 ] ; then
        shift
    fi
done

#=====================================================================
# Find a database so we can get the host list
#=====================================================================

if [ "$verbose" = "true" ] ; then
    echo "Creating host list"
fi

# Get Greenplum install major version 
gpversion=`postgres --gp-version`
productversion=`echo $gpversion | awk -F" " '{print $4}' | awk -F"." '{print $1}'`

if [ "$database" = "" ] ; then
    psql -l >/tmp/sas_dblist.txt

    i=1
    while read myline; do
       if [ $i -gt 3 ] ; then
          database=`echo $myline | awk -F"|" '{print $1}'`
          if [ "$verbose" = "true" ] ; then
              echo "Trying database $database"
          fi
          if [ $productversion -ge 4 ] ; then
              echo "select distinct hostname from gp_segment_configuration;" >/tmp/sas_cmds.txt
          else
              echo "select distinct hostname from gp_configuration;" >/tmp/sas_cmds.txt
          fi

          # query database for host list
          psql -d $database -f /tmp/sas_cmds.txt >/tmp/sas_hosts.txt 2>/tmp/sas_errors.txt
          if [ -s /tmp/sas_hosts.txt ] ; then
               break;
          fi
       fi
       let i++
    done </tmp/sas_dblist.txt

    rm /tmp/sas_dblist.txt

    if [ ! -s /tmp/sas_hosts.txt ] ; then
        echo "ERROR: could not find a valid database"
        if [ -e /tmp/sas_errors.txt ] ; then
            cat /tmp/sas_errors.txt
            rm /tmp/sas_errors.txt
        fi
        rm /tmp/sas_cmds.txt
        exit 1
    fi

else
    if [ "$verbose" = "true" ] ; then
        echo "Database is $database"
    fi

    if [ $productversion -ge 4 ] ; then
        echo "select distinct hostname from gp_segment_configuration;" >/tmp/sas_cmds.txt
    else
        echo "select distinct hostname from gp_configuration;" >/tmp/sas_cmds.txt
    fi

    # query database for host list
    psql -d $database -f /tmp/sas_cmds.txt >/tmp/sas_hosts.txt 2>/tmp/sas_errors.txt
    if [ ! -s /tmp/sas_hosts.txt ] ; then
        echo "ERROR: Invalid database specified"
        if [ -e /tmp/sas_errors.txt ] ; then
            cat /tmp/sas_errors.txt
            rm /tmp/sas_errors.txt
        fi
        rm /tmp/sas_cmds.txt
        exit 1
    fi
fi

if [ -e /tmp/sas_cmds.txt ] ; then
    rm /tmp/sas_cmds.txt
fi

if [ -e /tmp/sas_errors.txt ] ; then
  rm /tmp/sas_errors.txt
fi

#=====================================================================
# Parse the sas_hosts.txt file to find the host names.
#
# First two lines are "hostname" and "--------" and are ignored.
# Last line is the number of rows which are also ignored (4 rows).
# Remaining lines should indicate the hosts to add to the list.
#=====================================================================

hostlist=
hosts=

for f in `cat /tmp/sas_hosts.txt`; do
  if [ ! "$f" = "hostname" ] ; then
    echo $f | grep -q '\-\-\-\-'
    if [ ! $? = 0 ] ; then
      echo $f | grep -q '('
      if [ ! $? = 0 ] ; then
        echo $f | grep -q ')'
        if [ ! $? = 0 ] ; then
          hostlist="$hostlist -h $f"
          hosts="$hosts $f"
        fi
      fi
    fi
  fi
done

rm /tmp/sas_hosts.txt

if [ "$hostlist" = "" ] ; then
    echo "ERROR: could not determine host list"
    exit 2
fi

#=====================================================================
# Get the status information for the SAS EP process on each host 
#=====================================================================

if [ "$verbose" = "true" ] ; then
   echo "Host list is: $hosts"
fi

echo "SAS EP Process: $ep_process" 

for h in $hosts; do

   # Collect the process information using the PID read from the EP 
   # process PID file
   status="Down"
   pid=`ssh $h "if [ -e $pid_file ] ; then cat $pid_file; fi"`
   if [ "$pid" = "" ] ; then
       status="Not Started"
   else
       psout=`ssh $h "ps -p $pid -o ucmd,thcount,vsize,rss,pmem,pcpu,lstart | sed 1d"`
       if [ "$psout" != "" ] ; then
           ucmd=`echo $psout | awk -F" " '{print $1}'`
           if [ "$ucmd" = "$ep_process" ] ; then
               status="Up";
               thcount=`echo $psout | awk -F" " '{print $2}'`
               vsize=`echo $psout | awk -F" " '{print $3}'`
               rss=`echo $psout | awk -F" " '{print $4}'`
               pmem=`echo $psout | awk -F" " '{print $5}'`
               pcpu=`echo $psout | awk -F" " '{print $6}'`
               lstart=`echo $psout | awk -F" " '{print $7, $8, $9, $10, $11}'`
           fi
       fi
   fi

   # Display the process information
   echo "-------------------------------------------"
   echo "$h"
   echo "  Status:          $status"
   if [ "$status" = "Up" ] ; then
       echo "  PID:             $pid"
       echo "  Thread count:    $thcount"
       echo "  Virtual memory:  $vsize kb"
       echo "  Resident memory: $rss kb"
       echo "  %Memory:         $pmem"
       echo "  %CPU:            $pcpu"
       echo "  Started:         $lstart"
   fi

done

exit 0
