#!/bin/sh
#
# Shuts down the SAS EP process on each host of a Greenplum
# database. This script should be executed from the master
# database host node.
#
# Usage: 
#    ShutdownSASEP.sh [-quiet] [-verbose] [-database <database>]
#

quiet="false"
verbose="false"
database=""
mycommand=$0

pid_file="/var/tmp/tkgpmain.pid"
ep_process="tkgpmain"

product=SASEPForGreenplum
long_product="SAS Embedded Process for Greenplum"

#=====================================================================
# Handle passed in arguments
#=====================================================================

while [ "$#" -gt 0 ]
do
    if [ $1 = "-quiet" ] ; then
        quiet="true"
    elif [ $1 = "-verbose" ] ; then
        verbose="true"
    elif [ $1 = "-database" ] ; then
        shift
        if [ "$#" -gt 0 ] ; then
            database=$1
        fi
    fi
    if [ "$#" -gt 0 ] ; then
        shift
    fi
done

#=====================================================================
# Make sure the user wants to shut down the SAS EP process 
#=====================================================================

proceed="n"
if [ "$quiet" = "false" ] ; then
    echo "WARNING: The SAS EP process ($ep_process) should be shut down only " 
    echo "if no scoring is currently in progress."
    read -p "Do you want to proceed with the shutdown on all hosts (y/n)? " proceed 
    if [ ! "$proceed" == "y" ] && [ ! "$proceed" == "Y" ] ; then 
        echo "Exiting without shutting down the SAS EP process!" 
        exit 0
    fi
fi

#=====================================================================
# Find a database so we can get the host list
#=====================================================================

if [ "$verbose" = "true" ] ; then
    echo "Creating host list"
fi

# Get Greenplum install major version 
gpversion=`postgres --gp-version`
productversion=`echo $gpversion | awk -F" " '{print $4}' | awk -F"." '{print $1}'`

if [ "$database" = "" ] ; then
    psql -l >/tmp/sas_dblist.txt

    i=1
    while read myline; do
       if [ $i -gt 3 ] ; then
          database=`echo $myline | awk -F"|" '{print $1}'`
          if [ "$verbose" = "true" ] ; then
              echo "Trying database $database"
          fi
          if [ $productversion -ge 4 ] ; then
              echo "select distinct hostname from gp_segment_configuration;" >/tmp/sas_cmds.txt
          else
              echo "select distinct hostname from gp_configuration;" >/tmp/sas_cmds.txt
          fi

          # query database for host list
          psql -d $database -f /tmp/sas_cmds.txt >/tmp/sas_hosts.txt 2>/tmp/sas_errors.txt
          if [ -s /tmp/sas_hosts.txt ] ; then
               break;
          fi
       fi
       let i++
    done </tmp/sas_dblist.txt

    rm /tmp/sas_dblist.txt

    if [ ! -s /tmp/sas_hosts.txt ] ; then
        echo "ERROR: could not find a valid database"
        if [ -e /tmp/sas_errors.txt ] ; then
            cat /tmp/sas_errors.txt
            rm /tmp/sas_errors.txt
        fi
        rm /tmp/sas_cmds.txt
        exit 1
    fi

else
    if [ "$verbose" = "true" ] ; then
        echo "Database is $database"
    fi

    if [ $productversion -ge 4 ] ; then
        echo "select distinct hostname from gp_segment_configuration;" >/tmp/sas_cmds.txt
    else
        echo "select distinct hostname from gp_configuration;" >/tmp/sas_cmds.txt
    fi

    # query database for host list
    psql -d $database -f /tmp/sas_cmds.txt >/tmp/sas_hosts.txt 2>/tmp/sas_errors.txt
    if [ ! -s /tmp/sas_hosts.txt ] ; then
        echo "ERROR: Invalid database specified"
        if [ -e /tmp/sas_errors.txt ] ; then
            cat /tmp/sas_errors.txt
            rm /tmp/sas_errors.txt
        fi
        rm /tmp/sas_cmds.txt
        exit 1
    fi
fi

if [ -e /tmp/sas_cmds.txt ] ; then
    rm /tmp/sas_cmds.txt
fi

if [ -e /tmp/sas_errors.txt ] ; then
  rm /tmp/sas_errors.txt
fi

#=====================================================================
# Parse the sas_hosts.txt file to find the host names.
#
# First two lines are "hostname" and "--------" and are ignored.
# Last line is the number of rows which are also ignored (4 rows).
# Remaining lines should indicate the hosts to add to the list.
#=====================================================================

hostlist=
hosts=

for f in `cat /tmp/sas_hosts.txt`; do
  if [ ! "$f" = "hostname" ] ; then
    echo $f | grep -q '\-\-\-\-'
    if [ ! $? = 0 ] ; then
      echo $f | grep -q '('
      if [ ! $? = 0 ] ; then
        echo $f | grep -q ')'
        if [ ! $? = 0 ] ; then
          hostlist="$hostlist -h $f"
          hosts="$hosts $f"
        fi
      fi
    fi
  fi
done

rm /tmp/sas_hosts.txt

if [ "$verbose" = "true" ] ; then
    echo "Host list is: $hosts"
fi

if [ "$hostlist" = "" ] ; then
    echo "ERROR: could not determine host list"
    exit 2
fi

#=====================================================================
# Remove the SAS EP PID file on each host. 
# The SAS EP process monitors this file and shuts itself down
# if this file is removed.
#=====================================================================

if [ "$quiet" = "false" ] ; then
   echo "Initiating shutdown of the SAS EP process on all hosts"
fi

for h in $hosts; do
   if [ "$quiet" = "false" ] ; then
      echo "Stopping SAS EP process on $h"
   fi
   ssh $h "if [ -e $pid_file ] ; then rm $pid_file; fi"
   if [ ! "$?" -eq 0 ] ; then
       echo "ERROR: Could not remove $pid_file on $h"
   fi
done

if [ "$quiet" = "false" ] ; then
   echo "Waiting 30 seconds for the processes to shutdown..."
fi

# Give the EP time to shutdown 
sleep 30 

#=====================================================================
# Make sure the SAS EP process has shutdown on each host. 
# If the process did not shut itself down, then force it down.
#=====================================================================

if [ "$quiet" = "false" ] ; then
     echo "Verifying that the SAS EP process is shutdown on all hosts"
fi

for h in $hosts; do
   pidlist="";
   if [ "$quiet" = "false" ] ; then
       echo "Verifying SAS EP process is stopped on $h"
   fi
   psout=`ssh $h "ps -o ucmd,pid | grep $ep_process"`
   for f in $psout; do 
     if [ ! "$f" = "$ep_process" ] ; then 
        pidlist="$pidlist $f"
     fi
   done 

   if [ ! "$pidlist" = "" ] ; then 
       if [ "$quiet" = "false" ] ; then
           echo "Forcing process shutdown on $h"
       fi
       ssh $h "kill -9 $pidlist"
       if [ ! "$?" -eq 0 ] ; then
           echo "ERROR: Could not shutdown SAS EP process on $h"
           exit 3 
       fi
   fi

done

if [ "$quiet" = "false" ] ; then
   echo "SAS EP process shutdown completed!"
fi

exit 0
