#!/bin/sh
#
# Restarts the SAS EP process on each host of a Greenplum
# database. This script should be executed from the master
# database host node.
#
# Normally, the SAS EP process is started automatically
# when the SAS EP table function is invoked. The SAS EP
# process can also be started manually using this script.
#
# Before running this startup script, the ShutdownSASEP.sh
# script should be run to shutdown any running instances 
# of the SAS EP process. 
#
# Usage: 
#    StartupSASEP.sh [-quiet] [-verbose] [-database <database>]
#

quiet="false"
verbose="false"
database=""
mycommand=$0

pid_file="/var/tmp/tkgpmain.pid"
ep_process="tkgpmain"

product=SASEPForGreenplum
long_product="SAS Embedded Process for Greenplum"

#=====================================================================
# Handle passed in arguments
#=====================================================================

while [ "$#" -gt 0 ]
do
    if [ $1 = "-quiet" ] ; then
        quiet="true"
    elif [ $1 = "-verbose" ] ; then
        verbose="true"
    elif [ $1 = "-database" ] ; then
        shift
        if [ "$#" -gt 0 ] ; then
            database=$1
        fi
    fi
    if [ "$#" -gt 0 ] ; then
        shift
    fi
done

#=====================================================================
# Check with the user first 
#=====================================================================

proceed="n"
echo "WARNING: The SAS EP process ($ep_process) is normally started automatically "
echo "by the EP database table function whenever a score is executed and the "
echo "EP process is not already running. " 
echo "WARNING: Before running this startup script, please make sure that the " 
echo "ShutdownSASEP.sh script has been run to first shutdown the EP process " 
echo "on all hosts. " 

read -p "Do you want to proceed with the startup on all hosts (y/n)? " proceed
if [ ! "$proceed" == "y" ] && [ ! "$proceed" == "Y" ] ; then
     echo "Exiting without trying to start up the EP process!"
     exit 0
fi

#=====================================================================
# Find a database so we can get the host list
#=====================================================================

if [ "$verbose" = "true" ] ; then
    echo "Creating host list"
fi

# Get Greenplum install major version 
gpversion=`postgres --gp-version`
productversion=`echo $gpversion | awk -F" " '{print $4}' | awk -F"." '{print $1}'`

if [ "$database" = "" ] ; then
    psql -l >/tmp/sas_dblist.txt

    i=1
    while read myline; do
       if [ $i -gt 3 ] ; then
          database=`echo $myline | awk -F"|" '{print $1}'`
          if [ "$verbose" = "true" ] ; then
              echo "Trying database $database"
          fi
          if [ $productversion -ge 4 ] ; then
              echo "select distinct hostname from gp_segment_configuration;" >/tmp/sas_cmds.txt
          else
              echo "select distinct hostname from gp_configuration;" >/tmp/sas_cmds.txt
          fi

          # query database for host list
          psql -d $database -f /tmp/sas_cmds.txt >/tmp/sas_hosts.txt 2>/tmp/sas_errors.txt
          if [ -s /tmp/sas_hosts.txt ] ; then
               break;
          fi
       fi
       let i++
    done </tmp/sas_dblist.txt

    rm /tmp/sas_dblist.txt

    if [ ! -s /tmp/sas_hosts.txt ] ; then
        echo "ERROR: could not find a valid database"
        if [ -e /tmp/sas_errors.txt ] ; then
            cat /tmp/sas_errors.txt
            rm /tmp/sas_errors.txt
        fi
        rm /tmp/sas_cmds.txt
        exit 1
    fi

else
    if [ "$verbose" = "true" ] ; then
        echo "Database is $database"
    fi

    if [ $productversion -ge 4 ] ; then
        echo "select distinct hostname from gp_segment_configuration;" >/tmp/sas_cmds.txt
    else
        echo "select distinct hostname from gp_configuration;" >/tmp/sas_cmds.txt
    fi

    # query database for host list
    psql -d $database -f /tmp/sas_cmds.txt >/tmp/sas_hosts.txt 2>/tmp/sas_errors.txt
    if [ ! -s /tmp/sas_hosts.txt ] ; then
        echo "ERROR: Invalid database specified"
        if [ -e /tmp/sas_errors.txt ] ; then
            cat /tmp/sas_errors.txt
            rm /tmp/sas_errors.txt
        fi
        rm /tmp/sas_cmds.txt
        exit 1
    fi
fi

if [ -e /tmp/sas_cmds.txt ] ; then
    rm /tmp/sas_cmds.txt
fi

if [ -e /tmp/sas_errors.txt ] ; then
  rm /tmp/sas_errors.txt
fi

#=====================================================================
# Parse the sas_hosts.txt file to find the host names.
#
# First two lines are "hostname" and "--------" and are ignored.
# Last line is the number of rows which are also ignored (4 rows).
# Remaining lines should indicate the hosts to add to the list.
#=====================================================================

hostlist=
hosts=

for f in `cat /tmp/sas_hosts.txt`; do
  if [ ! "$f" = "hostname" ] ; then
    echo $f | grep -q '\-\-\-\-'
    if [ ! $? = 0 ] ; then
      echo $f | grep -q '('
      if [ ! $? = 0 ] ; then
        echo $f | grep -q ')'
        if [ ! $? = 0 ] ; then
          hostlist="$hostlist -h $f"
          hosts="$hosts $f"
        fi
      fi
    fi
  fi
done

rm /tmp/sas_hosts.txt

if [ "$verbose" = "true" ] ; then
    echo "Host list is: $hosts"
fi

if [ "$hostlist" = "" ] ; then
    echo "ERROR: could not determine host list"
    exit 2
fi

#=====================================================================
# Start the EP process on each host
#=====================================================================

bindir=`pg_config --bindir`

for h in $hosts; do
   if [ "$quiet" = "false" ] ; then
      echo "Starting the SAS EP process $bindir/tkgpmain on $h"
   fi
   if [ ! -e $bindir/tkgpmain ] ; then
      echo "ERROR: $bindir/tkgpmain not found on $h"
   else 
      #ssh -n -f "ssh -c $h "nohup $bindir/tkgpmain > /dev/null 2>&1 &\"
      ssh -n -f  $h "sh -c \"cd $bindir; nohup ./tkgpmain > /dev/null 2>&1 &\""
   fi
done

if [ "$quiet" = "false" ] ; then
   echo "SAS EP startup completed!"
fi

exit 0
